// ==============================================================
//                 ORBITER MODULE: spot_light
//
// spot_light.cpp
// Control module for spot_light vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================



class spot_light: public VESSEL3 {
public:
	spot_light (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel) {}

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);

	bool EnterRange(char *name1);
	bool EnterSpread(char *name2);
	bool EnterAttach(char *name3);


private:
	void SetSpotlight();
	int light_status;
	int beacon_count;
	double range, spread_angle;
	SpotLight *spotlight1;
	BEACONLIGHTSPEC spot_beacon[9];
	VECTOR3 spotcolour, beacon_col;
	VECTOR3 beacon_pos[9];
	double spc1, spc2, spc3;
	double intensity;
};
// Define animation sequences for moving parts
// --------------------------------------------------------------



// ==============================================================
// spot_light class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void spot_light::clbkSetClassCaps (FILEHANDLE cfg)
{

	intensity = 1;
	light_status = 0;
	range = 200;
	spread_angle = 20*RAD;
	beacon_count = 1;

	spc1 = 1;
	spc2 = 1;
	spc3 = 1;

	spotcolour = _V(0,0,0);
	if(oapiReadItem_vec (cfg, "SpotColour", spotcolour) == true)
	{
	if (spotcolour.x > 1)spotcolour.x = 1;
	else if (spotcolour.x < 0)spotcolour.x = 0;
	spc1 = spotcolour.x;

	if (spotcolour.y > 1)spotcolour.y = 1;
	else if (spotcolour.y < 0)spotcolour.y = 0;
	spc2 = spotcolour.y;

	if (spotcolour.z > 1)spotcolour.z = 1;
	else if (spotcolour.z < 0)spotcolour.z = 0;
	spc3 = spotcolour.z;
	}


	

	if(oapiReadItem_vec (cfg, "FlarePos1", beacon_pos[0]) == false) beacon_pos[0] = _V(0,0,0.01);
	if(oapiReadItem_vec (cfg, "FlarePos2", beacon_pos[1]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos3", beacon_pos[2]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos4", beacon_pos[3]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos5", beacon_pos[4]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos6", beacon_pos[5]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos7", beacon_pos[6]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos8", beacon_pos[7]) == true) beacon_count = beacon_count+1;
	if(oapiReadItem_vec (cfg, "FlarePos9", beacon_pos[8]) == true) beacon_count = beacon_count+1;
	
	
	
	beacon_col = _V(spc1,spc2,spc3);

	int ib;
	for (ib = 0;ib < beacon_count;ib++)
	{
		spot_beacon[ib].shape = BEACONSHAPE_STAR;
		spot_beacon[ib].pos = &beacon_pos[ib];
		spot_beacon[ib].col = &beacon_col;
		spot_beacon[ib].size = 0.28;
		spot_beacon[ib].falloff = 0.3;
		spot_beacon[ib].period = 0;
		spot_beacon[ib].duration = 0;
		spot_beacon[ib].tofs = 0;
		spot_beacon[ib].active = false;
		AddBeacon (&spot_beacon[ib]);

	}


	COLOUR4 col_d = {spc1,spc2,spc3,0};
	COLOUR4 col_s = {spc1,spc2,spc3,0};
	COLOUR4 col_a = {0,0,0,0};
	
	spotlight1 = (SpotLight*)AddSpotLight(_V(0,0,0.1), _V(0,0,1),200, 1, 0, 5e-5, 15*RAD, (20*RAD), col_d, col_s, col_a);
	spotlight1->Activate(false);

	SetVisibilityLimit(1e-4, -1);
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new spot_light (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (spot_light*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void spot_light::clbkPostCreation ()
{
SetSpotlight();
if(light_status == 1)
	{
	for (int i = 0; i < beacon_count; i++)
		{
		spot_beacon[i].active = true;
		}

	spotlight1->Activate(true);
	}



}
void spot_light::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters
	sprintf (cbuf, "%d %0.4f %0.4f %0.4f", light_status, range, spread_angle, intensity);
	oapiWriteScenario_string (scn, "SPT", cbuf);

}
void spot_light::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "SPT", 3)) {
			sscanf (line+3, "%d%lf%lf%lf", &light_status, &range, &spread_angle, &intensity);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
}

int spot_light::clbkConsumeDirectKey(char *kstate)
{
if (KEYDOWN (kstate, OAPI_KEY_J))
{
intensity = intensity - (0.2*oapiGetSimStep());
if (intensity < 0)intensity = 0;
spotlight1->SetIntensity(intensity);
for(int i = 0;i < beacon_count; i++)
	{
	spot_beacon[i].size = (sqrt(range))*0.02*intensity;
	}
}
else if (KEYDOWN (kstate, OAPI_KEY_K))
{
intensity = intensity + (0.2*oapiGetSimStep());
if (intensity > 1)intensity = 1;
spotlight1->SetIntensity(intensity);
for(int i = 0;i < beacon_count; i++)
	{
	spot_beacon[i].size = (sqrt(range))*0.02*intensity;
	}
}

return 0;
}


int spot_light::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	bool cbEnterRange(void *id1, char *str1, void *data1);
	bool cbEnterSpread(void *id2, char *str2, void *data2);
	bool cbEnterAttach(void *id3, char *str3, void *data3);



  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) {
	  switch (key)
	  {
		case OAPI_KEY_V: // detach from parent

		OBJHANDLE hParentVessel;
		ATTACHMENTHANDLE hChildAtt = GetAttachmentHandle(true, 0);
		if (hChildAtt) hParentVessel = GetAttachmentStatus(hChildAtt);
		if (hParentVessel)
		{
			VESSEL *v = oapiGetVesselInterface (hParentVessel);
			DWORD att_count = v->AttachmentCount (false);
			for (int i = 0; i < att_count; i++)
			{
			ATTACHMENTHANDLE hParentAttach = v->GetAttachmentHandle(false, i);
			OBJHANDLE hChildVessel = v->GetAttachmentStatus(hParentAttach);
			if (hChildVessel == GetHandle()) v->DetachChild(hParentAttach);
			}
		}
		return 1;
	  }
	  
  } else { // unmodified keys
    switch (key) {
	case OAPI_KEY_B: // spotlight on/off

		if(light_status == 0)
		{
			spotlight1->Activate(true);
			for (int i = 0; i < beacon_count; i++)
			{
			spot_beacon[i].active = true;
			}
			light_status = 1;
		}

		else if(light_status == 1)
		{
			spotlight1->Activate(false);
			for (int i = 0; i < beacon_count; i++)
			{
			spot_beacon[i].active = false;
			}
			light_status = 0;
		}

		return 1;

	case OAPI_KEY_N:
		{
			oapiOpenInputBox("Enter Range(m)", cbEnterRange, 0, 30, (void *)this);
		}
		return true;

	case OAPI_KEY_M:
		{
			oapiOpenInputBox("Enter Spread Angle(deg)", cbEnterSpread, 0, 30, (void *)this);
		}
		return true;

	case OAPI_KEY_V:
		{
			oapiOpenInputBox("Enter Parent Name & Att.Point Index", cbEnterAttach, 0, 40, (void *)this);
		}
		return true;

	}
  }
  return 0;
}

bool cbEnterRange(void *id1, char *str1, void *data1)
{
	return (((spot_light *)data1)->EnterRange(str1));
}

bool spot_light::EnterRange(char *Name1)
{
double range_chk = 0;

	if (Name1)
	{
	sscanf(Name1,"%lf", &range_chk);

	if (range_chk < 1)
		{
		range_chk = 1;
		range = range_chk;
		SetSpotlight();
		return true;
		}

	else if (range_chk > 100000)
		{
		range_chk = 100000;
		range = range_chk;
		SetSpotlight();
		return true;
		}
	else if (range_chk >= 1 && range_chk <= 100000)
		{
		range = range_chk;
		SetSpotlight();
		return true;
		}
	
	}
	return false;
}

bool cbEnterSpread(void *id2, char *str2, void *data2)
{
	return (((spot_light *)data2)->EnterSpread(str2));
}

bool spot_light::EnterSpread(char *Name2)
{
	if (Name2)
	{
	double spread_angle_chk = 0;
	sscanf(Name2,"%lf", &spread_angle_chk);

	if (spread_angle_chk < 10)
		{
		spread_angle_chk = 10;
		spread_angle = spread_angle_chk*RAD;
		SetSpotlight();
		return true;
		}

	else if (spread_angle_chk > 120)
		{
		spread_angle_chk = 120;
		spread_angle = spread_angle_chk*RAD;
		SetSpotlight();
		return true;
		}

	else if (spread_angle_chk >= 10 && spread_angle_chk <= 120)
		{
		spread_angle = spread_angle_chk*RAD;
		SetSpotlight();
		return true;
		}
	
	}
	return false;
}


bool cbEnterAttach(void *id3, char *str3, void *data3)
{
	return (((spot_light *)data3)->EnterAttach(str3));
}

bool spot_light::EnterAttach(char *Name3)
{

	ATTACHMENTHANDLE hPatt, hCatt;
	OBJHANDLE hPves, hDves, hSves;
	char vesname[256];
	DWORD att_index = 0;

	if (Name3)
	{
	sscanf(Name3,"%s%d", &vesname, &att_index);
	hPves = oapiGetObjectByName(vesname);
	if (!hPves) return false;
	else
		{
		VESSEL *v = oapiGetVesselInterface (hPves);
		DWORD nAttach = v->AttachmentCount (false);
		if (nAttach < (att_index+1))return false;
		else
			{
			hPatt = v->GetAttachmentHandle(false, att_index);
			hDves = v->GetAttachmentStatus(hPatt);
			if (hDves) return false;
			else
				{
				hSves = GetHandle();
				hCatt = GetAttachmentHandle(true, 0);
				if((v->AttachChild(hSves, hPatt, hCatt)) == true)return true;				
				}
			}

		}

	
	
	}
	return false;
}



void spot_light::clbkPreStep(double SimT, double SimDT, double MJD) 
{
	
}


void spot_light::clbkPostStep (double simt, double simdt, double mjd)
{
	
}

void spot_light::SetSpotlight()
{
	spotlight1->SetRange(range);
	spotlight1->SetAperture((spread_angle*0.75), (spread_angle));
	spotlight1->SetAttenuation(1,0, (2/(range*range)));
	spotlight1->SetIntensity(intensity);
	for (int i = 0; i < beacon_count; i++)
	{
	spot_beacon[i].size = (sqrt(range))*0.02*intensity;
	}
}

bool spot_light::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	oapiSetDefNavDisplay(0);
	oapiSetDefRCSDisplay(0);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);


	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);


	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[B] = Spotlight On/Off");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[N] = Enter Range %0.1fm", range);
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[M] = Enter Spread Angle %0.1f", (spread_angle*DEG));
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	int intpc = (int)(intensity*100);
	sprintf(abuf,"[J]/[K] = -/+ Intensity %d%%",intpc);
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"[V]/[Shift+V] = Easy Attach/Detach");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}


	return true;
}